#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, \
    IParagraphContainer, IParagraphRenderer
from pyams_content.component.illustration.interfaces import IIllustration, IIllustrationParagraph, \
    ILLUSTRATION_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from transaction.interfaces import ITransactionManager
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.illustration.paragraph import Illustration
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


#
# Illustration
#

@viewlet_config(name='add-illustration.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=60)
class IllustrationAddMenu(BaseParagraphAddMenu):
    """Illustration add menu"""

    label = _("Illustration...")
    label_css_class = 'fa fa-fw fa-file-image-o'
    url = 'add-illustration.html'
    paragraph_type = ILLUSTRATION_PARAGRAPH_TYPE


@pagelet_config(name='add-illustration.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class IllustrationAddForm(AdminDialogAddForm):
    """Illustration add form"""

    legend = _("Add new illustration")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-file-image-o'

    fields = field.Fields(IIllustrationParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-illustration.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(IllustrationAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return Illustration()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-illustration.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class IllustrationAJAXAddForm(BaseParagraphAJAXAddForm, IllustrationAddForm):
    """HTML paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IIllustrationParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class IllustrationPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Illustration properties edit form"""

    legend = _("Edit illustration properties")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-file-image-o'

    fields = field.Fields(IIllustrationParagraph).omit('__parent__', '__name__', 'visible')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(IllustrationPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IIllustrationParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class IllustrationPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, IllustrationPropertiesEditForm):
    """Illustration properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(IllustrationPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'title' in changes.get(IIllustration, ()):
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                    'object_name': self.context.__name__,
                    'title': II18n(self.context).query_attribute('title', request=self.request),
                    'visible': self.context.visible
                }
            })
        return output


@adapter_config(context=(IIllustrationParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class IllustrationInnerEditForm(IllustrationPropertiesEditForm):
    """Illustration inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IIllustrationParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class IllustrationInnerAJAXEditForm(BaseParagraphAJAXEditForm, IllustrationInnerEditForm):
    """Illustration paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(IllustrationInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IIllustration, ())
        if 'title' in updated:
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                    'object_name': self.context.__name__,
                    'title': II18n(self.context).query_attribute('title', request=self.request),
                    'visible': self.context.visible
                }
            })
        updated = changes.get(IIllustrationParagraph, ())
        if 'data' in updated:
            # we have to commit transaction to be able to handle blobs...
            ITransactionManager(self.context).get().commit()
            form = IllustrationInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'object_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'content': form.render()
                }
            })
        return output


#
# Illustration renderer
#

@adapter_config(context=(IIllustrationParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class IllustrationRenderer(BaseRenderedContentRenderer):
    """Illustration renderer"""
