#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationTarget
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphFactorySettings, \
    IParagraphContainer, IParagraphRenderer
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph, IRawParagraph, RAW_PARAGRAPH_TYPE, \
    HTML_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces import IPropertiesEditForm
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.association.zmi import AssociationsTable
from pyams_content.component.paragraph.html import HTMLParagraph, RawParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.component.paragraph.zmi.container import ParagraphContainerTable, \
    ParagraphTitleToolbarViewletManager
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuDivider
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, Interface

from pyams_content import _


#
# Raw HTML paragraph
#

@viewlet_config(name='add-raw-paragraph.divider', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=990)
class RawParagraphAddMenuDivider(MenuDivider):
    """Raw paragraph add menu divider"""

    def __new__(cls, context, request, view, manager):
        settings = get_parent(context, IParagraphFactorySettings)
        if (settings is not None) and (RawParagraphAddMenu.paragraph_type not in (settings.allowed_paragraphs or ())):
            return None
        return MenuDivider.__new__(cls)


@viewlet_config(name='add-raw-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=999)
class RawParagraphAddMenu(BaseParagraphAddMenu):
    """Raw HTML paragraph add menu"""

    label = _("Raw HTML...")
    label_css_class = 'fa fa-fw fa-code'
    url = 'add-raw-paragraph.html'
    paragraph_type = RAW_PARAGRAPH_TYPE


@pagelet_config(name='add-raw-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class RawParagraphAddForm(AdminDialogAddForm):
    """Raw HTML paragraph add form"""

    legend = _("Add new raw HTML paragraph")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-code'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IRawParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-raw-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(RawParagraphAddForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea height-100'

    def create(self, data):
        return RawParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-raw-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class RawParagraphAJAXAddForm(BaseParagraphAJAXAddForm, RawParagraphAddForm):
    """Raw HTML paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IRawParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class RawParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Raw HTML paragraph properties edit form"""

    legend = _("Edit raw HTML paragraph properties")
    icon_css_class = 'fa fa-fw fa-code'

    fields = field.Fields(IRawParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(RawParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea height-100'


@view_config(name='properties.json', context=IRawParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class RawParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, RawParagraphPropertiesEditForm):
    """Raw HTML paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IRawParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class RawParagraphInnerEditForm(RawParagraphPropertiesEditForm):
    """Raw HTML paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IRawParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class RawParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, RawParagraphInnerEditForm):
    """Raw HTML paragraph inner edit form, JSON renderer"""


#
# Raw HTML paragraph renderer
#

@adapter_config(context=(IRawParagraph, IPyAMSLayer), provides=IParagraphRenderer)
@template_config(template='templates/raw-render.pt', layer=IPyAMSLayer)
class RawParagraphRenderer(BaseContentProvider):
    """Raw HTML paragraph renderer"""

    language = None

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))


#
# Rich text paragraph
#

@viewlet_config(name='add-html-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=50)
class HTMLParagraphAddMenu(BaseParagraphAddMenu):
    """Rich text paragraph add menu"""

    label = _("Rich text...")
    label_css_class = 'fa fa-fw fa-html5'
    url = 'add-html-paragraph.html'
    paragraph_type = HTML_PARAGRAPH_TYPE


@pagelet_config(name='add-html-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphAddForm(AdminDialogAddForm):
    """Rich text paragraph add form"""

    legend = _("Add new rich text paragraph")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-html-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphAddForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea'

    def create(self, data):
        return HTMLParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-html-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphAJAXAddForm(BaseParagraphAJAXAddForm, HTMLParagraphAddForm):
    """Rich text paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IHTMLParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Rich text paragraph properties edit form"""

    legend = _("Edit rich text paragraph properties")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            body_widget = self.widgets['body']
            for lang in body_widget.langs:
                widget = body_widget.widgets[lang]
                widget.id = '{id}_{name}'.format(id=widget.id, name=self.context.__name__)
            body_widget.widget_css_class = 'textarea'


@view_config(name='properties.json', context=IHTMLParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, HTMLParagraphPropertiesEditForm):
    """Rich text paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HTMLParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'body' in changes.get(IHTMLParagraph, ()):
            # refresh associations count markers
            parent = get_parent(self.context, IAssociationTarget)
            table = ParagraphContainerTable(parent, self.request)
            viewlet = ParagraphTitleToolbarViewletManager(parent, self.request, table)
            viewlet.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.updateToolbar',
                    'object_name': self.context.__name__,
                    'toolbar_tag': viewlet.render()
                }
            })
            # refresh associations table
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.associations.refreshAssociations',
                    'object_id': associations_table.id,
                    'table': associations_table.render()
                }
            })
        return output


@adapter_config(context=(IHTMLParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IPropertiesEditForm, IAssociationsParentForm)
class HTMLParagraphInnerEditForm(HTMLParagraphPropertiesEditForm):
    """Rich text paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IHTMLParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, HTMLParagraphInnerEditForm):
    """Rich text paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HTMLParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        if 'body' in changes.get(IHTMLParagraph, ()):
            # refresh associations count markers
            parent = get_parent(self.context, IAssociationTarget)
            table = ParagraphContainerTable(parent, self.request)
            viewlet = ParagraphTitleToolbarViewletManager(parent, self.request, table)
            viewlet.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.updateToolbar',
                    'object_name': self.context.__name__,
                    'toolbar_tag': viewlet.render()
                }
            })
            # refresh associations table
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.associations.refreshAssociations',
                    'object_id': associations_table.id,
                    'table': associations_table.render()
                }
            })
        return output


#
# Rich text paragraph renderer
#

@adapter_config(context=(IHTMLParagraph, IPyAMSLayer), provides=IParagraphRenderer)
@template_config(template='templates/html-render.pt', layer=IPyAMSLayer)
class HTMLParagraphRenderer(BaseContentProvider):
    """Rich text paragraph renderer"""

    illustration = None
    illustration_renderer = None
    language = None

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))
        self.illustration = IIllustration(self.context)
        if self.illustration.data:
            renderer = self.illustration_renderer = self.illustration.get_renderer()
            if renderer is not None:
                renderer.update()

    def render_illustration(self):
        if not self.illustration_renderer:
            return ''
        return self.illustration_renderer.render()
